"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import collections.abc
import typing

import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import tensorflow.core.framework.tensor_pb2
import tensorflow.core.framework.tensor_shape_pb2
import tensorflow.core.framework.types_pb2

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class SnapshotRecord(google.protobuf.message.Message):
    """Each SnapshotRecord represents one batch of pre-processed input data. A batch
    consists of a list of tensors that we encode as TensorProtos. This message
    doesn't store the structure of the batch.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    TENSOR_FIELD_NUMBER: builtins.int
    @property
    def tensor(
        self,
    ) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[
        tensorflow.core.framework.tensor_pb2.TensorProto
    ]: ...
    def __init__(
        self, *, tensor: collections.abc.Iterable[tensorflow.core.framework.tensor_pb2.TensorProto] | None = ...
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["tensor", b"tensor"]) -> None: ...

global___SnapshotRecord = SnapshotRecord

@typing.final
class SnapshotMetadataRecord(google.protobuf.message.Message):
    """This stores the metadata information present in each snapshot record."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    GRAPH_HASH_FIELD_NUMBER: builtins.int
    RUN_ID_FIELD_NUMBER: builtins.int
    CREATION_TIMESTAMP_FIELD_NUMBER: builtins.int
    VERSION_FIELD_NUMBER: builtins.int
    DTYPE_FIELD_NUMBER: builtins.int
    NUM_ELEMENTS_FIELD_NUMBER: builtins.int
    FINALIZED_FIELD_NUMBER: builtins.int
    graph_hash: builtins.str
    """Stores the fingerprint of the graph that describes the dataset that is
    snapshotted.
    """
    run_id: builtins.str
    """Run ID that this snapshot corresponds to."""
    creation_timestamp: builtins.int
    """Time when we started creating this snapshot."""
    version: builtins.int
    """Version of the snapshot data file format."""
    num_elements: builtins.int
    """The number of elements in the snapshot."""
    finalized: builtins.bool
    @property
    def dtype(
        self,
    ) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[tensorflow.core.framework.types_pb2.DataType.ValueType]:
        """A list of tensor dtype corresponding to each element of the snapshot."""

    def __init__(
        self,
        *,
        graph_hash: builtins.str | None = ...,
        run_id: builtins.str | None = ...,
        creation_timestamp: builtins.int | None = ...,
        version: builtins.int | None = ...,
        dtype: collections.abc.Iterable[tensorflow.core.framework.types_pb2.DataType.ValueType] | None = ...,
        num_elements: builtins.int | None = ...,
        finalized: builtins.bool | None = ...,
    ) -> None: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "creation_timestamp",
            b"creation_timestamp",
            "dtype",
            b"dtype",
            "finalized",
            b"finalized",
            "graph_hash",
            b"graph_hash",
            "num_elements",
            b"num_elements",
            "run_id",
            b"run_id",
            "version",
            b"version",
        ],
    ) -> None: ...

global___SnapshotMetadataRecord = SnapshotMetadataRecord

@typing.final
class TensorMetadata(google.protobuf.message.Message):
    """Metadata for a single tensor in the Snapshot Record."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    TENSOR_SHAPE_FIELD_NUMBER: builtins.int
    TENSOR_SIZE_BYTES_FIELD_NUMBER: builtins.int
    tensor_size_bytes: builtins.int
    """Number of uncompressed bytes used to store the tensor representation."""
    @property
    def tensor_shape(self) -> tensorflow.core.framework.tensor_shape_pb2.TensorShapeProto: ...
    def __init__(
        self,
        *,
        tensor_shape: tensorflow.core.framework.tensor_shape_pb2.TensorShapeProto | None = ...,
        tensor_size_bytes: builtins.int | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["tensor_shape", b"tensor_shape"]) -> builtins.bool: ...
    def ClearField(
        self, field_name: typing.Literal["tensor_shape", b"tensor_shape", "tensor_size_bytes", b"tensor_size_bytes"]
    ) -> None: ...

global___TensorMetadata = TensorMetadata

@typing.final
class SnapshotTensorMetadata(google.protobuf.message.Message):
    """Metadata for all the tensors in a Snapshot Record."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    TENSOR_METADATA_FIELD_NUMBER: builtins.int
    @property
    def tensor_metadata(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___TensorMetadata]: ...
    def __init__(self, *, tensor_metadata: collections.abc.Iterable[global___TensorMetadata] | None = ...) -> None: ...
    def ClearField(self, field_name: typing.Literal["tensor_metadata", b"tensor_metadata"]) -> None: ...

global___SnapshotTensorMetadata = SnapshotTensorMetadata

@typing.final
class DistributedSnapshotMetadata(google.protobuf.message.Message):
    """Metadata for a `tf.data.Dataset` distributed snapshot."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ELEMENT_SPEC_FIELD_NUMBER: builtins.int
    COMPRESSION_FIELD_NUMBER: builtins.int
    element_spec: builtins.bytes
    """The element spec of the snapshotted dataset."""
    compression: builtins.str
    """Whether and how to compress the snapshot.  Supported values are defined in
    `tsl::io::compression`.  In particular, an empty string specifies not to
    compress.
    """
    def __init__(self, *, element_spec: builtins.bytes | None = ..., compression: builtins.str | None = ...) -> None: ...
    def ClearField(self, field_name: typing.Literal["compression", b"compression", "element_spec", b"element_spec"]) -> None: ...

global___DistributedSnapshotMetadata = DistributedSnapshotMetadata



from _typeshed import Incomplete

from authlib.oauth1.rfc5849.base_server import BaseServer

class AuthorizationServer(BaseServer):
    TOKEN_RESPONSE_HEADER: Incomplete
    TEMPORARY_CREDENTIALS_METHOD: str
    def create_oauth1_request(self, request) -> None: ...
    def handle_response(self, status_code, payload, headers) -> None: ...
    def handle_error_response(self, error): ...
    def validate_temporary_credentials_request(self, request): ...
    def create_temporary_credentials_response(self, request=None): ...
    def validate_authorization_request(self, request): ...
    def create_authorization_response(self, request, grant_user=None): ...
    def validate_token_request(self, request): ...
    def create_token_response(self, request): ...
    def create_temporary_credential(self, request) -> None: ...
    def get_temporary_credential(self, request) -> None: ...
    def delete_temporary_credential(self, request) -> None: ...
    def create_authorization_verifier(self, request) -> None: ...
    def create_token_credential(self, request) -> None: ...



"""Collection of passes for Jac IR.

This module uses lazy imports to enable converting passes to Jac.
The .jac passes are loaded lazily via __getattr__ to avoid circular imports.
"""

from __future__ import annotations

from typing import TYPE_CHECKING

# Passes that are imported lazily to allow .jac conversion
# These are loaded on first access via __getattr__
_LAZY_PASSES = {
    "CFGBuildPass": ".cfg_build_pass",
    "PyastBuildPass": ".pyast_load_pass",
    "PyJacAstLinkPass": ".pyjac_ast_link_pass",
    "SemDefMatchPass": ".sem_def_match_pass",
    "TypeCheckPass": ".type_checker_pass",
}

# Cache for lazily loaded passes
_lazy_cache: dict[str, type] = {}

if TYPE_CHECKING:
    from .cfg_build_pass import CFGBuildPass as CFGBuildPass
    from .pyast_load_pass import PyastBuildPass as PyastBuildPass
    from .pyjac_ast_link_pass import PyJacAstLinkPass as PyJacAstLinkPass
    from .sem_def_match_pass import SemDefMatchPass as SemDefMatchPass
    from .type_checker_pass import TypeCheckPass as TypeCheckPass


def __getattr__(name: str) -> type:
    """Lazily load passes on first access.

    All lazy passes are .jac files - Python passes are imported directly from pycore.
    """
    if name in _lazy_cache:
        return _lazy_cache[name]

    if name in _LAZY_PASSES:
        import importlib.util
        import os
        import sys

        module_name = _LAZY_PASSES[name]
        base_name = module_name.lstrip(".")

        # Load .jac file
        package_dir = os.path.dirname(__file__)
        jac_file = os.path.join(package_dir, f"{base_name}.jac")
        full_module_name = f"{__name__}.{base_name}"

        if os.path.exists(jac_file):
            # Use Jac import mechanism via the meta importer
            from jaclang.meta_importer import JacMetaImporter
            from jaclang.pycore.runtime import JacRuntime as Jac

            # Create module spec and load
            importer = JacMetaImporter()
            spec = importlib.util.spec_from_file_location(
                full_module_name, jac_file, loader=importer
            )
            if spec and spec.loader:
                module = importlib.util.module_from_spec(spec)
                sys.modules[full_module_name] = module
                Jac.load_module(full_module_name, module)
                spec.loader.exec_module(module)
            else:
                raise ImportError(f"Could not load Jac module: {jac_file}")
        else:
            raise ImportError(f"Jac module not found: {jac_file}")

        cls = getattr(module, name)
        _lazy_cache[name] = cls
        return cls

    raise AttributeError(f"module {__name__!r} has no attribute {name!r}")


__all__ = [
    "CFGBuildPass",
    "PyastBuildPass",
    "PyJacAstLinkPass",
    "SemDefMatchPass",
    "TypeCheckPass",
]



from typing import Literal

from pymeeus.Angle import Angle
from pymeeus.Epoch import Epoch

class Sun:
    def __init__(self) -> None: ...
    @staticmethod
    def true_longitude_coarse(epoch: Epoch) -> tuple[Angle, float]: ...
    @staticmethod
    def apparent_longitude_coarse(epoch: Epoch) -> tuple[Angle, float]: ...
    @staticmethod
    def apparent_rightascension_declination_coarse(epoch: Epoch) -> tuple[Angle, Angle, float]: ...
    @staticmethod
    def geometric_geocentric_position(epoch: Epoch, tofk5: bool = True) -> tuple[Angle, Angle, float]: ...
    @staticmethod
    def apparent_geocentric_position(epoch: Epoch, nutation: bool = True) -> tuple[Angle, Angle, float]: ...
    @staticmethod
    def rectangular_coordinates_mean_equinox(epoch: Epoch) -> tuple[float, float, float]: ...
    @staticmethod
    def rectangular_coordinates_j2000(epoch: Epoch) -> tuple[float, float, float]: ...
    @staticmethod
    def rectangular_coordinates_b1950(epoch: Epoch) -> tuple[float, float, float]: ...
    @staticmethod
    def rectangular_coordinates_equinox(epoch: Epoch, equinox_epoch: Epoch) -> tuple[float, float, float]: ...
    @staticmethod
    def get_equinox_solstice(year: int, target: Literal["spring", "summer", "autumn", "winter"] = "spring") -> Epoch: ...
    @staticmethod
    def equation_of_time(epoch: Epoch) -> tuple[int, float]: ...
    @staticmethod
    def ephemeris_physical_observations(epoch: Epoch) -> tuple[Angle, Angle, Angle]: ...
    @staticmethod
    def beginning_synodic_rotation(number: int) -> Epoch: ...

def main() -> None: ...



from typing import Final, Literal

EPOCH: Final = 1922501.5
EPOCH_SARKAWAG: Final = 2117210.5
MONTHS: Final[list[str]]
MONTHS_ARM: Final[list[str]]

def leap(year: int) -> bool: ...
def to_jd(year: int, month: int, day: int, method: Literal["sarkawag", "moveable"] | None = None) -> float: ...
def from_jd(jd: float, method: Literal["sarkawag", "moveable"] | None = None) -> tuple[int, int, int]: ...
def to_julian(year: int, month: int, day: int, method: Literal["sarkawag", "moveable"] | None = None) -> tuple[int, int, int]: ...
def from_julian(
    year: int, month: int, day: int, method: Literal["sarkawag", "moveable"] | None = None
) -> tuple[int, int, int]: ...
def to_gregorian(
    year: int, month: int, day: int, method: Literal["sarkawag", "moveable"] | None = None
) -> tuple[int, int, int]: ...
def from_gregorian(
    year: int, month: int, day: int, method: Literal["sarkawag", "moveable"] | None = None
) -> tuple[int, int, int]: ...
def month_length(year: int, month: int, method: Literal["sarkawag", "moveable"] | None = None) -> Literal[5, 6, 30]: ...
def monthcalendar(year: int, month: int, method: Literal["sarkawag", "moveable"] | None = None) -> list[list[int | None]]: ...
def format(year: int, month: int, day: int, lang: str | None = None) -> str: ...
def tostring(year: int, month: int, day: int, lang: str | None = None) -> str: ...



import argparse
import ast
from collections.abc import Generator, Iterable
from typing import Any, ClassVar, Final, Literal
from typing_extensions import Self

from flake8.options.manager import OptionManager

__version__: Final[str]
__all__ = ("pep257Checker",)

class pep257Checker:
    name: ClassVar[str]
    version: ClassVar[str]
    tree: ast.AST
    filename: str
    checker: Any  # actual type: pep257.ConventionChecker
    source: str
    def __init__(self, tree: ast.AST, filename: str, lines: Iterable[str]) -> None: ...
    @classmethod
    def add_options(cls, parser: OptionManager) -> None: ...
    @classmethod
    def parse_options(cls, options: argparse.Namespace) -> None: ...
    def run(self) -> Generator[tuple[int, Literal[0], str, type[Self]]]: ...



from __future__ import annotations

import subprocess
import sys
from collections.abc import Iterable
from http.client import HTTPResponse
from pathlib import Path
from typing import TYPE_CHECKING
from urllib.request import urlopen
from zipfile import ZipFile

from mypy_protobuf.main import (  # type: ignore[import-untyped]  # pyright: ignore[reportMissingTypeStubs]
    __version__ as mypy_protobuf__version__,
)

if TYPE_CHECKING:
    from _typeshed import StrOrBytesPath, StrPath

MYPY_PROTOBUF_VERSION = mypy_protobuf__version__


def download_file(url: str, destination: Path) -> None:
    print(f"Downloading '{url}' to '{destination}'")
    resp: HTTPResponse
    with urlopen(url) as resp:
        destination.write_bytes(resp.read())


def extract_archive(archive_path: StrPath, destination: StrPath) -> None:
    print(f"Extracting '{archive_path}' to '{destination}'")
    with ZipFile(archive_path) as file_in:
        file_in.extractall(destination)


def run_protoc(
    proto_paths: Iterable[StrPath], mypy_out: StrPath, proto_globs: Iterable[str], cwd: StrOrBytesPath | None = None
) -> str:
    """TODO: Describe parameters and return."""
    protoc_version = (
        subprocess.run([sys.executable, "-m", "grpc_tools.protoc", "--version"], capture_output=True, check=False)
        .stdout.decode()
        .strip()
    )
    print()
    print(protoc_version)
    protoc_args = [
        *[f"--proto_path={proto_path}" for proto_path in proto_paths],
        "--mypy_out",
        f"relax_strict_optional_primitives:{mypy_out}",
        *proto_globs,
    ]
    print("Running: protoc\n    " + "\n    ".join(protoc_args) + "\n")
    subprocess.run((sys.executable, "-m", "grpc_tools.protoc", *protoc_args), cwd=cwd, check=True)
    return protoc_version



from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = ["non_randomness"]

@_dispatchable
def non_randomness(G: Graph[_Node], k: int | None = None, weight: str | None = "weight"): ...



from collections.abc import Generator

from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = ["generate_multiline_adjlist", "write_multiline_adjlist", "parse_multiline_adjlist", "read_multiline_adjlist"]

def generate_multiline_adjlist(G: Graph[_Node], delimiter: str = " ") -> Generator[str, None, None]: ...
def write_multiline_adjlist(G: Graph[_Node], path, delimiter=" ", comments="#", encoding="utf-8") -> None: ...
@_dispatchable
def parse_multiline_adjlist(lines, comments: str = "#", delimiter=None, create_using=None, nodetype=None, edgetype=None): ...
@_dispatchable
def read_multiline_adjlist(
    path, comments: str = "#", delimiter=None, create_using=None, nodetype=None, edgetype=None, encoding: str = "utf-8"
): ...



from _typeshed import StrPath, SupportsKeysAndGetItem
from collections.abc import Container, Iterable, Iterator, Mapping, MutableMapping, Sequence
from typing import Literal, TypeVar, overload
from typing_extensions import TypeAlias
from uuid import UUID

_T = TypeVar("_T")

_Token: TypeAlias = (
    tuple[Literal["EMPTY"], str, None]
    | tuple[Literal["COMMENT"], str, None]
    | tuple[Literal["SECTION"], str, tuple[str, ...]]
    | tuple[Literal["KV"], str, tuple[str, str, str]]
)

def get_app_dir(app_name: str, roaming: bool = ..., force_posix: bool = ...) -> str: ...

class Dialect:
    def __init__(
        self,
        ns_sep: str = ...,
        kv_sep: str = ...,
        quotes: Sequence[str] = ...,
        true: Sequence[str] = ...,
        false: Sequence[str] = ...,
        comments: Container[str] = ...,
        allow_escaping: bool = ...,
        linesep: str | None = ...,
    ) -> None: ...
    @property
    def ns_sep(self) -> str: ...
    @property
    def kv_sep(self) -> str: ...
    @property
    def quotes(self) -> Sequence[str]: ...
    @property
    def true(self) -> Sequence[str]: ...
    @property
    def false(self) -> Sequence[str]: ...
    @property
    def comments(self) -> Container[str]: ...
    @property
    def allow_escaping(self) -> bool: ...
    @property
    def linesep(self) -> str | None: ...
    def get_actual_linesep(self) -> str: ...
    def get_strippable_lineseps(self) -> str: ...
    def kv_serialize(self, key: str, val: str | None) -> str | None: ...
    def escape(self, value: str, quote: str | None = ...) -> str: ...
    def unescape(self, value: str) -> str: ...
    def to_string(self, value: bool | float | str) -> str: ...
    def dict_from_iterable(self, iterable: Iterable[str]) -> MutableMapping[str, str]: ...
    def tokenize(self, iterable: Iterable[str]) -> Iterator[_Token]: ...
    def update_tokens(
        self, old_tokens: Iterable[_Token], changes: SupportsKeysAndGetItem[str, str] | Iterable[tuple[str, str]]
    ) -> list[_Token]: ...

default_dialect: Dialect

class IniData(MutableMapping[str, str]):
    def __init__(self, mapping: Mapping[str, str] | None = ..., dialect: Dialect | None = ...) -> None: ...
    @property
    def dialect(self) -> Dialect: ...
    @property
    def is_dirty(self) -> bool: ...
    def get_updated_lines(self, line_iter: Iterable[_Token] | None = ...) -> list[_Token]: ...
    def discard(self) -> None: ...
    def rollover(self) -> None: ...
    def to_dict(self) -> dict[str, str]: ...
    def __len__(self) -> int: ...
    @overload
    def get(self, name: str, default: None = None) -> str | None: ...
    @overload
    def get(self, name: str, default: str) -> str: ...
    @overload
    def get(self, name: str, default: _T) -> str | _T: ...
    @overload
    def get_ascii(self, name: str) -> str | None: ...
    @overload
    def get_ascii(self, name: str, default: _T) -> str | _T: ...
    @overload
    def get_bool(self, name: str) -> bool: ...
    @overload
    def get_bool(self, name: str, default: _T) -> bool | _T: ...
    @overload
    def get_int(self, name: str) -> int | None: ...
    @overload
    def get_int(self, name: str, default: _T = ...) -> int | _T: ...
    @overload
    def get_float(self, name: str) -> float | None: ...
    @overload
    def get_float(self, name: str, default: _T) -> float | _T: ...
    @overload
    def get_uuid(self, name: str) -> UUID | None: ...
    @overload
    def get_uuid(self, name: str, default: _T) -> UUID | _T: ...
    def itersections(self) -> Iterator[str]: ...
    def sections(self) -> Iterator[str]: ...
    def iteritems(self) -> Iterator[tuple[str, str]]: ...
    def iterkeys(self) -> Iterator[str]: ...
    def itervalues(self) -> Iterator[str]: ...
    # NB: keys, items, values currently return a generator, which is
    # incompatible with the views returned by Mappings
    def items(self) -> Iterator[tuple[str, str]]: ...  # type: ignore[override]
    def keys(self) -> Iterator[str]: ...  # type: ignore[override]
    def __iter__(self) -> Iterator[str]: ...
    def values(self) -> Iterator[str]: ...  # type: ignore[override]
    def section_as_dict(self, section: str) -> dict[str, str]: ...
    def __getitem__(self, name: str) -> str: ...
    def __setitem__(self, name: str, value: str) -> None: ...
    def __delitem__(self, name: str) -> None: ...

class IniFile(IniData):
    def __init__(self, filename: StrPath, encoding: str | None = ..., dialect: Dialect | None = ...) -> None: ...
    @property
    def filename(self) -> str: ...
    @property
    def encoding(self) -> str | None: ...
    @property
    def is_new(self) -> bool: ...
    def save(self, create_folder: bool = ...) -> None: ...

class AppIniFile(IniFile):
    def __init__(
        self,
        app_name: str,
        filename: StrPath,
        roaming: bool = ...,
        force_posix: bool = ...,
        encoding: str | None = ...,
        dialect: Dialect | None = ...,
    ) -> None: ...



import sys

if sys.version_info >= (3, 11):
    from .resources.simple import (
        ResourceContainer as ResourceContainer,
        ResourceHandle as ResourceHandle,
        SimpleReader as SimpleReader,
        TraversableReader as TraversableReader,
    )

    __all__ = ["SimpleReader", "ResourceHandle", "ResourceContainer", "TraversableReader"]



from .context import CryptContext

__all__ = [
    "custom_app_context",
    "django_context",
    "ldap_context",
    "ldap_nocrypt_context",
    "mysql_context",
    "mysql4_context",
    "mysql3_context",
    "phpass_context",
    "phpbb3_context",
    "postgres_context",
]

master_context: CryptContext
custom_app_context: CryptContext
django10_context: CryptContext
django14_context: CryptContext
django16_context: CryptContext
django110_context: CryptContext
django21_context: CryptContext
django_context = django21_context
std_ldap_schemes: list[str]
ldap_nocrypt_context: CryptContext
ldap_context: CryptContext
mysql3_context: CryptContext
mysql4_context: CryptContext
mysql_context = mysql4_context
postgres_context: CryptContext
phpass_context: CryptContext
phpbb3_context: CryptContext
roundup10_context: CryptContext
roundup15_context: CryptContext
roundup_context = roundup15_context



#!/usr/bin/env python3
"""
Generates the protobuf stubs for the given tensorflow version using mypy-protobuf.
Generally, new minor versions are a good time to update the stubs.
"""

from __future__ import annotations

import re
import shutil
import subprocess
import sys
import tempfile
from pathlib import Path

from _utils import MYPY_PROTOBUF_VERSION, download_file, extract_archive, run_protoc
from ts_utils.metadata import read_metadata, update_metadata
from ts_utils.paths import distribution_path

PACKAGE_VERSION = read_metadata("tensorflow").version_spec.version

STUBS_FOLDER = distribution_path("tensorflow").absolute()
ARCHIVE_FILENAME = f"v{PACKAGE_VERSION}.zip"
ARCHIVE_URL = f"https://github.com/tensorflow/tensorflow/archive/refs/tags/{ARCHIVE_FILENAME}"
EXTRACTED_PACKAGE_DIR = f"tensorflow-{PACKAGE_VERSION}"

PROTOS_TO_REMOVE = (
    "compiler/xla/autotune_results_pb2.pyi",
    "compiler/xla/autotuning_pb2.pyi",
    "compiler/xla/service/buffer_assignment_pb2.pyi",
    "compiler/xla/service/hlo_execution_profile_data_pb2.pyi",
    "core/protobuf/autotuning_pb2.pyi",
    "core/protobuf/conv_autotuning_pb2.pyi",
    "core/protobuf/critical_section_pb2.pyi",
    "core/protobuf/eager_service_pb2.pyi",
    "core/protobuf/master_pb2.pyi",
    "core/protobuf/master_service_pb2.pyi",
    "core/protobuf/replay_log_pb2.pyi",
    "core/protobuf/tpu/compile_metadata_pb2.pyi",
    "core/protobuf/worker_pb2.pyi",
    "core/protobuf/worker_service_pb2.pyi",
    "core/util/example_proto_fast_parsing_test_pb2.pyi",
)
"""
These protos exist in a folder with protos used in python,
but are not included in the python wheel.
They are likely only used for other language builds.
stubtest was used to identify them by looking for ModuleNotFoundError.
(comment out ".*_pb2.*" from the allowlist)
"""

TSL_IMPORT_PATTERN = re.compile(r"(\[|\s)tsl\.")
XLA_IMPORT_PATTERN = re.compile(r"(\[|\s)xla\.")


def move_tree(source: Path, destination: Path) -> None:
    """Move directory and merge if destination already exists.

    Can't use shutil.move because it can't merge existing directories.
    """
    print(f"Moving '{source}' to '{destination}'")
    shutil.copytree(source, destination, dirs_exist_ok=True)
    shutil.rmtree(source)


def post_creation() -> None:
    """Move third-party and fix imports."""
    print()
    move_tree(STUBS_FOLDER / "tsl", STUBS_FOLDER / "tensorflow" / "tsl")
    move_tree(STUBS_FOLDER / "xla", STUBS_FOLDER / "tensorflow" / "compiler" / "xla")

    for path in STUBS_FOLDER.rglob("*_pb2.pyi"):
        print(f"Fixing imports in '{path}'")
        filedata = path.read_text(encoding="utf-8")

        # Replace the target string
        filedata = re.sub(TSL_IMPORT_PATTERN, "\\1tensorflow.tsl.", filedata)
        filedata = re.sub(XLA_IMPORT_PATTERN, "\\1tensorflow.compiler.xla.", filedata)

        # Write the file out again
        path.write_text(filedata, encoding="utf-8")

    print()
    for to_remove in PROTOS_TO_REMOVE:
        file_path = STUBS_FOLDER / "tensorflow" / to_remove
        file_path.unlink()
        print(f"Removed '{file_path}'")


def main() -> None:
    temp_dir = Path(tempfile.mkdtemp())
    # Fetch tensorflow (which contains all the .proto files)
    archive_path = temp_dir / ARCHIVE_FILENAME
    download_file(ARCHIVE_URL, archive_path)
    extract_archive(archive_path, temp_dir)

    # Remove existing pyi
    for old_stub in STUBS_FOLDER.rglob("*_pb2.pyi"):
        old_stub.unlink()

    protoc_version = run_protoc(
        proto_paths=(
            f"{EXTRACTED_PACKAGE_DIR}/third_party/xla/third_party/tsl",
            f"{EXTRACTED_PACKAGE_DIR}/third_party/xla",
            f"{EXTRACTED_PACKAGE_DIR}",
        ),
        mypy_out=STUBS_FOLDER,
        proto_globs=(
            f"{EXTRACTED_PACKAGE_DIR}/third_party/xla/xla/*.proto",
            f"{EXTRACTED_PACKAGE_DIR}/third_party/xla/xla/service/*.proto",
            f"{EXTRACTED_PACKAGE_DIR}/third_party/xla/xla/tsl/protobuf/*.proto",
            f"{EXTRACTED_PACKAGE_DIR}/tensorflow/core/example/*.proto",
            f"{EXTRACTED_PACKAGE_DIR}/tensorflow/core/framework/*.proto",
            f"{EXTRACTED_PACKAGE_DIR}/tensorflow/core/protobuf/*.proto",
            f"{EXTRACTED_PACKAGE_DIR}/tensorflow/core/protobuf/tpu/*.proto",
            f"{EXTRACTED_PACKAGE_DIR}/tensorflow/core/util/*.proto",
            f"{EXTRACTED_PACKAGE_DIR}/tensorflow/python/keras/protobuf/*.proto",
            f"{EXTRACTED_PACKAGE_DIR}/third_party/xla/third_party/tsl/tsl/protobuf/*.proto",
        ),
        cwd=temp_dir,
    )

    # Cleanup after ourselves, this is a temp dir, but it can still grow fast if run multiple times
    shutil.rmtree(temp_dir)

    post_creation()

    update_metadata(
        "tensorflow",
        extra_description=f"""Partially generated using \
[mypy-protobuf=={MYPY_PROTOBUF_VERSION}](https://github.com/nipunn1313/mypy-protobuf/tree/v{MYPY_PROTOBUF_VERSION}) \
and {protoc_version} on `tensorflow=={PACKAGE_VERSION}`.""",
    )
    print("Updated tensorflow/METADATA.toml")

    # Run pre-commit to cleanup the stubs
    subprocess.run((sys.executable, "-m", "pre_commit", "run", "--files", *STUBS_FOLDER.rglob("*_pb2.pyi")), check=False)


if __name__ == "__main__":
    main()



from _typeshed import Incomplete
from typing import NoReturn

def RemoveCR(text): ...

SCRIPTTEXT_FORCEEXECUTION: int
SCRIPTTEXT_ISEXPRESSION: int
SCRIPTTEXT_ISPERSISTENT: int
state_map: Incomplete

def profile(fn, *args): ...

class SafeOutput:
    softspace: int
    redir: Incomplete
    def __init__(self, redir=None) -> None: ...
    def write(self, message) -> None: ...
    def flush(self) -> None: ...
    def close(self) -> None: ...

def MakeValidSysOuts() -> None: ...
def trace(*args) -> None: ...
def RaiseAssert(scode, desc) -> NoReturn: ...

class AXScriptCodeBlock:
    name: Incomplete
    codeText: Incomplete
    codeObject: Incomplete
    sourceContextCookie: Incomplete
    startLineNumber: Incomplete
    flags: Incomplete
    beenExecuted: int
    def __init__(self, name: str, codeText: str, sourceContextCookie: int, startLineNumber: int, flags) -> None: ...
    def GetFileName(self): ...
    def GetDisplayName(self): ...
    def GetLineNo(self, no: int): ...

class Event:
    name: str
    def __init__(self) -> None: ...
    def Reset(self) -> None: ...
    def Close(self) -> None: ...
    dispid: Incomplete
    def Build(self, typeinfo, funcdesc) -> None: ...

class EventSink:
    events: Incomplete
    connection: Incomplete
    coDispatch: Incomplete
    myScriptItem: Incomplete
    myInvokeMethod: Incomplete
    iid: Incomplete
    def __init__(self, myItem, coDispatch) -> None: ...
    def Reset(self) -> None: ...
    def Close(self) -> None: ...
    def GetSourceTypeInfo(self, typeinfo): ...
    def BuildEvents(self) -> None: ...
    def Connect(self) -> None: ...
    def Disconnect(self) -> None: ...

class ScriptItem:
    parentItem: Incomplete
    dispatch: Incomplete
    name: Incomplete
    flags: Incomplete
    eventSink: Incomplete
    subItems: Incomplete
    createdConnections: int
    isRegistered: int
    def __init__(self, parentItem, name, dispatch, flags) -> None: ...
    def Reset(self) -> None: ...
    def Close(self) -> None: ...
    def Register(self) -> None: ...
    def IsGlobal(self): ...
    def IsVisible(self): ...
    def GetEngine(self): ...
    def GetSubItemClass(self): ...
    def GetSubItem(self, name): ...
    def GetCreateSubItem(self, parentItem, name, dispatch, flags): ...
    def CreateConnections(self) -> None: ...
    def Connect(self) -> None: ...
    def Disconnect(self) -> None: ...
    def BuildEvents(self) -> None: ...
    def FindBuildSubItemEvents(self) -> None: ...
    def GetDefaultSourceTypeInfo(self, typeinfo): ...

IActiveScriptMethods: Incomplete
IActiveScriptParseMethods: Incomplete
IObjectSafetyMethods: Incomplete
IActiveScriptParseProcedureMethods: Incomplete

class COMScript:
    baseThreadId: int
    debugManager: Incomplete
    threadState: Incomplete
    scriptState: Incomplete
    scriptSite: Incomplete
    safetyOptions: int
    lcid: int
    subItems: Incomplete
    scriptCodeBlocks: Incomplete
    def __init__(self) -> None: ...
    def InitNew(self) -> None: ...
    def AddScriptlet(
        self, defaultName, code, itemName, subItemName, eventName, delimiter, sourceContextCookie, startLineNumber
    ) -> None: ...
    def ParseScriptText(self, code, itemName, context, delimiter, sourceContextCookie, startLineNumber, flags, bWantResult): ...
    def ParseProcedureText(
        self, code, formalParams, procName, itemName, unkContext, delimiter, contextCookie, startingLineNumber, flags
    ) -> None: ...
    def SetScriptSite(self, site) -> None: ...
    def GetScriptSite(self, iid): ...
    def SetScriptState(self, state) -> None: ...
    def GetScriptState(self): ...
    persistLoaded: int
    def Close(self) -> None: ...
    def AddNamedItem(self, name, flags) -> None: ...
    def GetScriptDispatch(self, name) -> None: ...
    def GetCurrentScriptThreadID(self): ...
    def GetScriptThreadID(self, win32ThreadId): ...
    def GetScriptThreadState(self, scriptThreadId): ...
    def AddTypeLib(self, uuid, major, minor, flags) -> None: ...
    def Clone(self) -> None: ...
    def SetInterfaceSafetyOptions(self, iid, optionsMask, enabledOptions) -> None: ...
    def GetInterfaceSafetyOptions(self, iid): ...
    def ExecutePendingScripts(self) -> None: ...
    def ProcessScriptItemEvent(self, item, event, lcid, wFlags, args): ...
    def ResetNamedItems(self) -> None: ...
    def GetCurrentSafetyOptions(self): ...
    def ProcessNewNamedItemsConnections(self) -> None: ...
    def RegisterNewNamedItems(self) -> None: ...
    def RegisterNamedItem(self, item) -> None: ...
    def CheckConnectedOrDisconnected(self) -> None: ...
    def Connect(self) -> None: ...
    def Run(self) -> None: ...
    def Stop(self) -> None: ...
    def Disconnect(self) -> None: ...
    def ConnectEventHandlers(self) -> None: ...
    def DisconnectEventHandlers(self) -> None: ...
    def Reset(self) -> None: ...
    def ChangeScriptState(self, state) -> None: ...
    def ApplyInScriptedSection(self, codeBlock: AXScriptCodeBlock | None, fn, args): ...
    def CompileInScriptedSection(self, codeBlock: AXScriptCodeBlock, type, realCode=None): ...
    def ExecInScriptedSection(self, codeBlock: AXScriptCodeBlock, globals, locals=None): ...
    def EvalInScriptedSection(self, codeBlock, globals, locals=None): ...
    def HandleException(self, codeBlock: AXScriptCodeBlock | None) -> NoReturn: ...
    def BeginScriptedSection(self) -> None: ...
    def EndScriptedSection(self) -> None: ...
    def DisableInterrupts(self) -> None: ...
    def EnableInterrupts(self) -> None: ...
    def GetNamedItem(self, name): ...
    def GetNamedItemClass(self): ...

def dumptypeinfo(typeinfo) -> None: ...



def patch() -> None: ...

class SyncInstrumentedTransport:
    def __init__(self, transport): ...
    def handle_request(self, request): ...

class AsyncInstrumentedTransport:
    def __init__(self, transport): ...
    async def handle_async_request(self, request): ...



from _typeshed import Incomplete
from logging import Logger

from oauthlib.common import _HTTPMethod

from .base import BaseEndpoint

log: Logger

class AuthorizationEndpoint(BaseEndpoint):
    def __init__(self, default_response_type: str, default_token_type: str, response_types: dict[str, Incomplete]) -> None: ...
    @property
    def response_types(self) -> dict[str, Incomplete]: ...
    @property
    def default_response_type(self) -> str: ...
    @property
    def default_response_type_handler(self): ...
    @property
    def default_token_type(self): ...
    def create_authorization_response(
        self,
        uri: str,
        http_method: _HTTPMethod = "GET",
        body: str | None = None,
        headers: dict[str, str] | None = None,
        scopes=None,
        credentials: dict[str, Incomplete] | None = None,
    ): ...
    def validate_authorization_request(
        self, uri: str, http_method: _HTTPMethod = "GET", body: str | None = None, headers: dict[str, str] | None = None
    ): ...



import socket
from collections.abc import Callable, Iterable, Iterator
from typing import ClassVar

from gunicorn.config import Config
from gunicorn.http.message import Request
from gunicorn.http.unreader import Unreader

from .._types import _AddressType

class Parser:
    mesg_class: ClassVar[type[Request] | None]
    cfg: Config
    unreader: Unreader
    mesg: Request | None
    source_addr: _AddressType
    req_count: int

    def __init__(self, cfg: Config, source: socket.socket | Iterable[bytes], source_addr: _AddressType) -> None: ...
    def __iter__(self) -> Iterator[Request]: ...
    def __next__(self) -> Request: ...

    next: Callable[[Parser], Request]

class RequestParser(Parser):
    mesg_class: ClassVar[type[Request]]



from typing import TypedDict

version_json: str
_Versions = TypedDict("_Versions", {"date": str, "dirty": bool, "error": None, "full-revisionid": str, "version": str})

def get_versions() -> _Versions: ...



from tensorflow import Tensor
from tensorflow._aliases import DTypeLike, TensorCompatible

def hamming_window(
    window_length: TensorCompatible, periodic: bool | TensorCompatible = True, dtype: DTypeLike = ..., name: str | None = None
) -> Tensor: ...



class ObjectDef:
    def __init__(self, object_class=None, schema=None, custom_validator=None, auxiliary_class=None) -> None: ...
    def __getitem__(self, item): ...
    def __getattr__(self, item: str): ...
    def __setattr__(self, key: str, value) -> None: ...
    def __iadd__(self, other): ...
    def __isub__(self, other): ...
    def __iter__(self): ...
    def __len__(self) -> int: ...
    def __bool__(self) -> bool: ...
    def __contains__(self, item): ...
    def add_from_schema(self, attribute_name, mandatory: bool = False) -> None: ...
    def add_attribute(self, definition=None) -> None: ...
    def remove_attribute(self, item) -> None: ...
    def clear_attributes(self) -> None: ...



from _typeshed import Incomplete
from collections.abc import Callable
from typing_extensions import deprecated

from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = ["could_be_isomorphic", "fast_could_be_isomorphic", "faster_could_be_isomorphic", "is_isomorphic"]

@_dispatchable
def could_be_isomorphic(G1: Graph[_Node], G2: Graph[_Node], *, properties: str = "dtc") -> bool: ...
@deprecated("`graph_could_be_isomorphic` is a deprecated alias for `could_be_isomorphic`. Use `could_be_isomorphic` instead.")
def graph_could_be_isomorphic(G1: Graph[_Node], G2: Graph[_Node]) -> bool: ...
@_dispatchable
def fast_could_be_isomorphic(G1: Graph[_Node], G2: Graph[_Node]) -> bool: ...
@deprecated(
    "`fast_graph_could_be_isomorphic` is a deprecated alias for `fast_could_be_isomorphic`. "
    "Use `fast_could_be_isomorphic` instead."
)
def fast_graph_could_be_isomorphic(G1: Graph[_Node], G2: Graph[_Node]) -> bool: ...
@_dispatchable
def faster_could_be_isomorphic(G1: Graph[_Node], G2: Graph[_Node]) -> bool: ...
@deprecated(
    "`faster_graph_could_be_isomorphic` is a deprecated alias for `faster_could_be_isomorphic`. "
    "Use `faster_could_be_isomorphic` instead."
)
def faster_graph_could_be_isomorphic(G1: Graph[_Node], G2: Graph[_Node]) -> bool: ...
@_dispatchable
def is_isomorphic(
    G1: Graph[_Node],
    G2: Graph[_Node],
    node_match: Callable[..., Incomplete] | None = None,
    edge_match: Callable[..., Incomplete] | None = None,
) -> bool: ...



import xml.etree.ElementTree as default_etree
from _typeshed import Incomplete, Unused
from collections.abc import Iterable, Mapping, Sequence
from typing import Final, TypeVar, overload

__all__ = [
    "default_etree",
    "MethodDispatcher",
    "isSurrogatePair",
    "surrogatePairToCodepoint",
    "moduleFactoryFactory",
    "supports_lone_surrogates",
]

supports_lone_surrogates: Final[bool]

_K = TypeVar("_K")
_V = TypeVar("_V")

class MethodDispatcher(dict[_K, _V]):
    default: _V | None
    @overload  # to solve `reportInvalidTypeVarUse`
    def __init__(self) -> None: ...
    @overload
    def __init__(self, items: Iterable[tuple[_K | Iterable[_K], _V]]) -> None: ...
    def __getitem__(self, key: _K) -> _V | None: ...  # type: ignore[override]
    def __get__(self, instance, owner: Unused = None) -> BoundMethodDispatcher: ...

class BoundMethodDispatcher(Mapping[Incomplete, Incomplete]):
    instance: Incomplete
    dispatcher: Incomplete
    def __init__(self, instance, dispatcher) -> None: ...
    def __getitem__(self, key): ...
    def get(self, key, default): ...  # type: ignore[override]
    def __iter__(self): ...
    def __len__(self) -> int: ...
    def __contains__(self, key) -> bool: ...

def isSurrogatePair(data: Sequence[str | bytes | bytearray]) -> bool: ...
def surrogatePairToCodepoint(data: Sequence[str | bytes | bytearray]) -> int: ...
def moduleFactoryFactory(factory): ...



from __future__ import annotations

from threading import local
from typing import Any, Callable

from attrs import frozen


@frozen
class AttributeOverride:
    omit_if_default: bool | None = None
    rename: str | None = None
    omit: bool | None = None  # Omit the field completely.
    struct_hook: Callable[[Any, Any], Any] | None = None  # Structure hook to use.
    unstruct_hook: Callable[[Any], Any] | None = None  # Structure hook to use.


neutral = AttributeOverride()
already_generating = local()



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import typing

import google.protobuf.descriptor
import google.protobuf.message
import tensorflow.core.framework.tensor_pb2

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class NamedTensorProto(google.protobuf.message.Message):
    """A pair of tensor name and tensor values."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    TENSOR_FIELD_NUMBER: builtins.int
    name: builtins.str
    """Name of the tensor."""
    @property
    def tensor(self) -> tensorflow.core.framework.tensor_pb2.TensorProto:
        """The client can populate a TensorProto using a tensorflow::Tensor`, or
        directly using the protobuf field accessors.

        The client specifies whether the returned tensor values should be
        filled tensor fields (float_val, int_val, etc.) or encoded in a
        compact form in tensor.tensor_content.
        """

    def __init__(
        self, *, name: builtins.str | None = ..., tensor: tensorflow.core.framework.tensor_pb2.TensorProto | None = ...
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["tensor", b"tensor"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["name", b"name", "tensor", b"tensor"]) -> None: ...

global___NamedTensorProto = NamedTensorProto



class ClientMixin:
    def get_default_redirect_uri(self) -> None: ...
    def get_client_secret(self) -> None: ...
    def get_rsa_public_key(self) -> None: ...

class TokenCredentialMixin:
    def get_oauth_token(self) -> None: ...
    def get_oauth_token_secret(self) -> None: ...

class TemporaryCredentialMixin(TokenCredentialMixin):
    def get_client_id(self) -> None: ...
    def get_redirect_uri(self) -> None: ...
    def check_verifier(self, verifier) -> None: ...

class TemporaryCredential(dict[str, object], TemporaryCredentialMixin):
    def get_client_id(self): ...
    def get_user_id(self): ...
    def get_redirect_uri(self): ...
    def check_verifier(self, verifier): ...
    def get_oauth_token(self): ...
    def get_oauth_token_secret(self): ...



from collections.abc import Callable, Iterable
from typing import Any, Literal

from matplotlib.axes import Axes
from matplotlib.typing import ColorType, LineStyleType, MarkerType

from ._core.typing import ColumnName, DataSource, Default, NormSpec
from .axisgrid import FacetGrid
from .external.kde import _BwMethodType
from .utils import _DataSourceWideForm, _ErrorBar, _Estimator, _Legend, _LogScale, _Palette, _Seed, _Vector

__all__ = ["catplot", "stripplot", "swarmplot", "boxplot", "violinplot", "boxenplot", "pointplot", "barplot", "countplot"]

def boxplot(
    data: DataSource | _DataSourceWideForm | None = None,
    *,
    x: ColumnName | _Vector | None = None,
    y: ColumnName | _Vector | None = None,
    hue: ColumnName | _Vector | None = None,
    order: Iterable[ColumnName] | None = None,
    hue_order: Iterable[ColumnName] | None = None,
    orient: Literal["v", "h", "x", "y"] | None = None,
    color: ColorType | None = None,
    palette: _Palette | None = None,
    saturation: float = 0.75,
    fill: bool = True,
    dodge: bool | Literal["auto"] = "auto",
    width: float = 0.8,
    gap: float = 0,
    whis: float = 1.5,
    linecolor: ColorType = "auto",
    linewidth: float | None = None,
    fliersize: float | None = None,
    hue_norm: NormSpec = None,
    native_scale: bool = False,
    log_scale: _LogScale | None = None,
    formatter: Callable[[Any], str] | None = None,
    legend: _Legend = "auto",
    ax: Axes | None = None,
    **kwargs: Any,
) -> Axes: ...
def violinplot(
    data: DataSource | _DataSourceWideForm | None = None,
    *,
    x: ColumnName | _Vector | None = None,
    y: ColumnName | _Vector | None = None,
    hue: ColumnName | _Vector | None = None,
    order: Iterable[ColumnName] | None = None,
    hue_order: Iterable[ColumnName] | None = None,
    orient: Literal["v", "h", "x", "y"] | None = None,
    color: ColorType | None = None,
    palette: _Palette | None = None,
    saturation: float = 0.75,
    fill: bool = True,
    inner: str | None = "box",
    split: bool = False,
    width: float = 0.8,
    dodge: bool | Literal["auto"] = "auto",
    gap: float = 0,
    linewidth: float | None = None,
    linecolor: ColorType = "auto",
    cut: float = 2,
    gridsize: int = 100,
    bw_method: _BwMethodType = "scott",
    bw_adjust: float = 1,
    density_norm: Literal["area", "count", "width"] = "area",
    common_norm: bool | None = False,
    hue_norm: NormSpec = None,
    formatter: Callable[[Any], str] | None = None,
    log_scale: _LogScale | None = None,
    native_scale: bool = False,
    legend: _Legend = "auto",
    scale=...,  # deprecated
    scale_hue=...,  # deprecated
    bw=...,  # deprecated
    inner_kws: dict[str, Any] | None = None,
    ax: Axes | None = None,
    **kwargs: Any,
) -> Axes: ...
def boxenplot(
    data: DataSource | _DataSourceWideForm | None = None,
    *,
    x: ColumnName | _Vector | None = None,
    y: ColumnName | _Vector | None = None,
    hue: ColumnName | _Vector | None = None,
    order: Iterable[ColumnName] | None = None,
    hue_order: Iterable[ColumnName] | None = None,
    orient: Literal["v", "h", "x", "y"] | None = None,
    color: ColorType | None = None,
    palette: _Palette | None = None,
    saturation: float = 0.75,
    fill: bool = True,
    dodge: bool | Literal["auto"] = "auto",
    width: float = 0.8,
    gap: float = 0,
    linewidth: float | None = None,
    linecolor: ColorType | None = None,
    width_method: Literal["exponential", "linear", "area"] = "exponential",
    k_depth: Literal["tukey", "proportion", "trustworthy", "full"] | int = "tukey",
    outlier_prop: float = 0.007,
    trust_alpha: float = 0.05,
    showfliers: bool = True,
    hue_norm: NormSpec = None,
    log_scale: _LogScale | None = None,
    native_scale: bool = False,
    formatter: Callable[[Any], str] | None = None,
    legend: _Legend = "auto",
    scale=...,  # deprecated
    box_kws: dict[str, Any] | None = None,
    flier_kws: dict[str, Any] | None = None,
    line_kws: dict[str, Any] | None = None,
    ax: Axes | None = None,
    **kwargs: Any,
) -> Axes: ...
def stripplot(
    data: DataSource | _DataSourceWideForm | None = None,
    *,
    x: ColumnName | _Vector | None = None,
    y: ColumnName | _Vector | None = None,
    hue: ColumnName | _Vector | None = None,
    order: Iterable[ColumnName] | None = None,
    hue_order: Iterable[ColumnName] | None = None,
    jitter: float | Literal[True] = True,
    dodge: bool = False,
    orient: Literal["v", "h", "x", "y"] | None = None,
    color: ColorType | None = None,
    palette: _Palette | None = None,
    size: float = 5,
    edgecolor: ColorType | Default = ...,
    linewidth: float = 0,
    hue_norm: NormSpec = None,
    log_scale: _LogScale | None = None,
    native_scale: bool = False,
    formatter: Callable[[Any], str] | None = None,
    legend: _Legend = "auto",
    ax: Axes | None = None,
    **kwargs: Any,
) -> Axes: ...
def swarmplot(
    data: DataSource | _DataSourceWideForm | None = None,
    *,
    x: ColumnName | _Vector | None = None,
    y: ColumnName | _Vector | None = None,
    hue: ColumnName | _Vector | None = None,
    order: Iterable[ColumnName] | None = None,
    hue_order: Iterable[ColumnName] | None = None,
    dodge: bool = False,
    orient: Literal["v", "h", "x", "y"] | None = None,
    color: ColorType | None = None,
    palette: _Palette | None = None,
    size: float = 5,
    edgecolor: ColorType | None = None,
    linewidth: float = 0,
    hue_norm: NormSpec = None,
    log_scale: _LogScale | None = None,
    native_scale: bool = False,
    formatter: Callable[[Any], str] | None = None,
    legend: _Legend = "auto",
    warn_thresh: float = 0.05,
    ax: Axes | None = None,
    **kwargs: Any,
) -> Axes: ...
def barplot(
    data: DataSource | _DataSourceWideForm | None = None,
    *,
    x: ColumnName | _Vector | None = None,
    y: ColumnName | _Vector | None = None,
    hue: ColumnName | _Vector | None = None,
    order: Iterable[ColumnName] | None = None,
    hue_order: Iterable[ColumnName] | None = None,
    estimator: _Estimator = "mean",
    errorbar: _ErrorBar | None = ("ci", 95),
    n_boot: int = 1000,
    units: ColumnName | _Vector | None = None,
    weights: ColumnName | _Vector | None = None,
    seed: _Seed | None = None,
    orient: Literal["v", "h", "x", "y"] | None = None,
    color: ColorType | None = None,
    palette: _Palette | None = None,
    saturation: float = 0.75,
    fill: bool = True,
    hue_norm: NormSpec = None,
    width: float = 0.8,
    dodge: bool | Literal["auto"] = "auto",
    gap: float = 0,
    log_scale: _LogScale | None = None,
    native_scale: bool = False,
    formatter: Callable[[Any], str] | None = None,
    legend: _Legend = "auto",
    capsize: float = 0,
    err_kws: dict[str, Any] | None = None,
    ci=...,  # deprecated
    errcolor=...,  # deprecated
    errwidth=...,  # deprecated
    ax: Axes | None = None,
    **kwargs: Any,
) -> Axes: ...
def pointplot(
    data: DataSource | _DataSourceWideForm | None = None,
    *,
    x: ColumnName | _Vector | None = None,
    y: ColumnName | _Vector | None = None,
    hue: ColumnName | _Vector | None = None,
    order: Iterable[ColumnName] | None = None,
    hue_order: Iterable[ColumnName] | None = None,
    estimator: _Estimator = "mean",
    errorbar: _ErrorBar | None = ("ci", 95),
    n_boot: int = 1000,
    units: ColumnName | _Vector | None = None,
    weights: ColumnName | _Vector | None = None,
    seed: _Seed | None = None,
    color: ColorType | None = None,
    palette: _Palette | None = None,
    hue_norm: NormSpec = None,
    markers: MarkerType | list[MarkerType] | Default = ...,
    linestyles: LineStyleType | list[LineStyleType] | Default = ...,
    dodge: bool = False,
    log_scale: _LogScale | None = None,
    native_scale: bool = False,
    orient: Literal["v", "h", "x", "y"] | None = None,
    capsize: float = 0,
    formatter: Callable[[Any], str] | None = None,
    legend: _Legend = "auto",
    err_kws: dict[str, Any] | None = None,
    ci=...,  # deprecated
    errwidth=...,  # deprecated
    join=...,  # deprecated
    scale=...,  # deprecated
    ax: Axes | None = None,
    **kwargs: Any,
) -> Axes: ...
def countplot(
    data: DataSource | _DataSourceWideForm | None = None,
    *,
    x: ColumnName | _Vector | None = None,
    y: ColumnName | _Vector | None = None,
    hue: ColumnName | _Vector | None = None,
    order: Iterable[ColumnName] | None = None,
    hue_order: Iterable[ColumnName] | None = None,
    orient: Literal["v", "h", "x", "y"] | None = None,
    color: ColorType | None = None,
    palette: _Palette | None = None,
    saturation: float = 0.75,
    fill: bool = True,
    hue_norm: NormSpec = None,
    stat: Literal["count", "percent", "proportion", "probability"] = "count",
    width: float = 0.8,
    dodge: bool | Literal["auto"] = "auto",
    gap: float = 0,
    log_scale: _LogScale | None = None,
    native_scale: bool = False,
    formatter: Callable[[Any], str] | None = None,
    legend: _Legend = "auto",
    ax: Axes | None = None,
    **kwargs: Any,
) -> Axes: ...
def catplot(
    data: DataSource | _DataSourceWideForm | None = None,
    *,
    x: ColumnName | _Vector | None = None,
    y: ColumnName | _Vector | None = None,
    hue: ColumnName | _Vector | None = None,
    row: ColumnName | _Vector | None = None,
    col: ColumnName | _Vector | None = None,
    kind: Literal["strip", "swarm", "box", "violin", "boxen", "point", "bar", "count"] = "strip",
    estimator: _Estimator = "mean",
    errorbar: _ErrorBar | None = ("ci", 95),
    n_boot: int = 1000,
    units: ColumnName | _Vector | None = None,
    weights: ColumnName | _Vector | None = None,
    seed: _Seed | None = None,
    order: Iterable[ColumnName] | None = None,
    hue_order: Iterable[ColumnName] | None = None,
    row_order: Iterable[ColumnName] | None = None,
    col_order: Iterable[ColumnName] | None = None,
    col_wrap: int | None = None,
    height: float = 5,
    aspect: float = 1,
    log_scale: _LogScale | None = None,
    native_scale: bool = False,
    formatter: Callable[[Any], str] | None = None,
    orient: Literal["v", "h", "x", "y"] | None = None,
    color: ColorType | None = None,
    palette: _Palette | None = None,
    hue_norm: NormSpec = None,
    legend: _Legend = "auto",
    legend_out: bool = True,
    sharex: bool = True,
    sharey: bool = True,
    margin_titles: bool = False,
    facet_kws: dict[str, Any] | None = None,
    ci=...,  # deprecated
    **kwargs: Any,
) -> FacetGrid: ...



from typing import ClassVar, Final, TypeVar

from docutils.parsers.rst import states
from docutils.readers import standalone

__docformat__: Final = "reStructuredText"

_S = TypeVar("_S", bound=str | bytes)

class Reader(standalone.Reader[_S]):
    settings_default_overrides: ClassVar[dict[str, int]]
    inliner_class: ClassVar[type[states.Inliner]]



from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = ["dispersion"]

@_dispatchable
def dispersion(
    G: Graph[_Node],
    u: _Node | None = None,
    v: _Node | None = None,
    normalized: bool = True,
    alpha: float = 1.0,
    b: float = 0.0,
    c: float = 0.0,
) -> dict[_Node, float] | dict[_Node, dict[_Node, float]]: ...



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import collections.abc
import sys
import typing

import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class _NodeClass:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _NodeClassEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_NodeClass.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    UNKNOWN: _NodeClass.ValueType  # 0
    INTERLEAVE_MANY: _NodeClass.ValueType  # 1
    ASYNC_INTERLEAVE_MANY: _NodeClass.ValueType  # 2
    KNOWN_RATIO: _NodeClass.ValueType  # 3
    ASYNC_KNOWN_RATIO: _NodeClass.ValueType  # 4
    UNKNOWN_RATIO: _NodeClass.ValueType  # 5
    ASYNC_UNKNOWN_RATIO: _NodeClass.ValueType  # 6

class NodeClass(_NodeClass, metaclass=_NodeClassEnumTypeWrapper):
    """Class of a node in the performance model."""

UNKNOWN: NodeClass.ValueType  # 0
INTERLEAVE_MANY: NodeClass.ValueType  # 1
ASYNC_INTERLEAVE_MANY: NodeClass.ValueType  # 2
KNOWN_RATIO: NodeClass.ValueType  # 3
ASYNC_KNOWN_RATIO: NodeClass.ValueType  # 4
UNKNOWN_RATIO: NodeClass.ValueType  # 5
ASYNC_UNKNOWN_RATIO: NodeClass.ValueType  # 6
global___NodeClass = NodeClass

class _AutotuneAlgorithm:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _AutotuneAlgorithmEnumTypeWrapper(
    google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_AutotuneAlgorithm.ValueType], builtins.type
):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    DEFAULT: _AutotuneAlgorithm.ValueType  # 0
    HILL_CLIMB: _AutotuneAlgorithm.ValueType  # 1
    GRADIENT_DESCENT: _AutotuneAlgorithm.ValueType  # 2
    MAX_PARALLELISM: _AutotuneAlgorithm.ValueType  # 3
    STAGE_BASED: _AutotuneAlgorithm.ValueType  # 4

class AutotuneAlgorithm(_AutotuneAlgorithm, metaclass=_AutotuneAlgorithmEnumTypeWrapper):
    """Algorithm used for model autotuning optimization."""

DEFAULT: AutotuneAlgorithm.ValueType  # 0
HILL_CLIMB: AutotuneAlgorithm.ValueType  # 1
GRADIENT_DESCENT: AutotuneAlgorithm.ValueType  # 2
MAX_PARALLELISM: AutotuneAlgorithm.ValueType  # 3
STAGE_BASED: AutotuneAlgorithm.ValueType  # 4
global___AutotuneAlgorithm = AutotuneAlgorithm

@typing.final
class ModelProto(google.protobuf.message.Message):
    """Protocol buffer representing the data used by the autotuning modeling
    framework.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class Node(google.protobuf.message.Message):
        """General representation of a node in the model."""

        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        @typing.final
        class Parameter(google.protobuf.message.Message):
            """Represents a node parameter."""

            DESCRIPTOR: google.protobuf.descriptor.Descriptor

            NAME_FIELD_NUMBER: builtins.int
            VALUE_FIELD_NUMBER: builtins.int
            STATE_VALUE_FIELD_NUMBER: builtins.int
            MIN_FIELD_NUMBER: builtins.int
            MAX_FIELD_NUMBER: builtins.int
            TUNABLE_FIELD_NUMBER: builtins.int
            name: builtins.str
            """Human-readable name of the parameter."""
            value: builtins.float
            """Identifies the model value of the parameter. This can be different from
            the actual value (e.g. during optimization search).
            """
            state_value: builtins.float
            """The actual value of the parameter."""
            min: builtins.float
            """Minimum value of the parameter."""
            max: builtins.float
            """Maximum value of the parameter."""
            tunable: builtins.bool
            """Identifies whether the parameter should participate in autotuning."""
            def __init__(
                self,
                *,
                name: builtins.str | None = ...,
                value: builtins.float | None = ...,
                state_value: builtins.float | None = ...,
                min: builtins.float | None = ...,
                max: builtins.float | None = ...,
                tunable: builtins.bool | None = ...,
            ) -> None: ...
            def ClearField(
                self,
                field_name: typing.Literal[
                    "max",
                    b"max",
                    "min",
                    b"min",
                    "name",
                    b"name",
                    "state_value",
                    b"state_value",
                    "tunable",
                    b"tunable",
                    "value",
                    b"value",
                ],
            ) -> None: ...

        ID_FIELD_NUMBER: builtins.int
        NAME_FIELD_NUMBER: builtins.int
        AUTOTUNE_FIELD_NUMBER: builtins.int
        BUFFERED_BYTES_FIELD_NUMBER: builtins.int
        BUFFERED_ELEMENTS_FIELD_NUMBER: builtins.int
        BYTES_CONSUMED_FIELD_NUMBER: builtins.int
        BYTES_PRODUCED_FIELD_NUMBER: builtins.int
        NUM_ELEMENTS_FIELD_NUMBER: builtins.int
        PROCESSING_TIME_FIELD_NUMBER: builtins.int
        RECORD_METRICS_FIELD_NUMBER: builtins.int
        PARAMETERS_FIELD_NUMBER: builtins.int
        INPUT_PROCESSING_TIME_SUM_FIELD_NUMBER: builtins.int
        INPUT_PROCESSING_TIME_COUNT_FIELD_NUMBER: builtins.int
        INPUTS_FIELD_NUMBER: builtins.int
        NODE_CLASS_FIELD_NUMBER: builtins.int
        RATIO_FIELD_NUMBER: builtins.int
        MEMORY_RATIO_FIELD_NUMBER: builtins.int
        id: builtins.int
        """Unique node ID."""
        name: builtins.str
        """Human-readable name of the node."""
        autotune: builtins.bool
        """An indication whether autotuning is enabled for this node."""
        buffered_bytes: builtins.int
        """The number of bytes stored in this node's buffer."""
        buffered_elements: builtins.int
        """The number of elements stored in this node's buffer."""
        bytes_consumed: builtins.int
        """The number of bytes consumed by the node."""
        bytes_produced: builtins.int
        """The number of bytes produced by the node."""
        num_elements: builtins.int
        """The number of elements produced by the node."""
        processing_time: builtins.int
        """The aggregate processing time spent in this node in nanoseconds."""
        record_metrics: builtins.bool
        """An indication whether this node records metrics about produced and
        consumed elements.
        """
        input_processing_time_sum: builtins.float
        """Statistic of inputs processing time history."""
        input_processing_time_count: builtins.int
        node_class: global___NodeClass.ValueType
        """Class of this node."""
        ratio: builtins.float
        """Ratio of input to output elements. This is only used by KNOWN_RATIO and
        ASYNC_KNOWN_RATIO nodes.
        """
        memory_ratio: builtins.float
        """Ratio identifies how many parallelism calls are introduced by one
        buffered element. This is only used by ASYNC_KNOWN_RATIO nodes.
        """
        @property
        def parameters(
            self,
        ) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___ModelProto.Node.Parameter]:
            """Parameters of this node."""

        @property
        def inputs(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]:
            """IDs of inputs of this node."""

        def __init__(
            self,
            *,
            id: builtins.int | None = ...,
            name: builtins.str | None = ...,
            autotune: builtins.bool | None = ...,
            buffered_bytes: builtins.int | None = ...,
            buffered_elements: builtins.int | None = ...,
            bytes_consumed: builtins.int | None = ...,
            bytes_produced: builtins.int | None = ...,
            num_elements: builtins.int | None = ...,
            processing_time: builtins.int | None = ...,
            record_metrics: builtins.bool | None = ...,
            parameters: collections.abc.Iterable[global___ModelProto.Node.Parameter] | None = ...,
            input_processing_time_sum: builtins.float | None = ...,
            input_processing_time_count: builtins.int | None = ...,
            inputs: collections.abc.Iterable[builtins.int] | None = ...,
            node_class: global___NodeClass.ValueType | None = ...,
            ratio: builtins.float | None = ...,
            memory_ratio: builtins.float | None = ...,
        ) -> None: ...
        def ClearField(
            self,
            field_name: typing.Literal[
                "autotune",
                b"autotune",
                "buffered_bytes",
                b"buffered_bytes",
                "buffered_elements",
                b"buffered_elements",
                "bytes_consumed",
                b"bytes_consumed",
                "bytes_produced",
                b"bytes_produced",
                "id",
                b"id",
                "input_processing_time_count",
                b"input_processing_time_count",
                "input_processing_time_sum",
                b"input_processing_time_sum",
                "inputs",
                b"inputs",
                "memory_ratio",
                b"memory_ratio",
                "name",
                b"name",
                "node_class",
                b"node_class",
                "num_elements",
                b"num_elements",
                "parameters",
                b"parameters",
                "processing_time",
                b"processing_time",
                "ratio",
                b"ratio",
                "record_metrics",
                b"record_metrics",
            ],
        ) -> None: ...

    @typing.final
    class NodesEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.int
        @property
        def value(self) -> global___ModelProto.Node: ...
        def __init__(self, *, key: builtins.int | None = ..., value: global___ModelProto.Node | None = ...) -> None: ...
        def HasField(self, field_name: typing.Literal["value", b"value"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing.Literal["key", b"key", "value", b"value"]) -> None: ...

    @typing.final
    class OptimizationParams(google.protobuf.message.Message):
        """Contains parameters of the model autotuning optimization."""

        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        ALGORITHM_FIELD_NUMBER: builtins.int
        CPU_BUDGET_FIELD_NUMBER: builtins.int
        RAM_BUDGET_FIELD_NUMBER: builtins.int
        MODEL_INPUT_TIME_FIELD_NUMBER: builtins.int
        algorithm: global___AutotuneAlgorithm.ValueType
        """Algorithm used for autotuning optimization."""
        cpu_budget: builtins.int
        """Number of available logical threads."""
        ram_budget: builtins.int
        """Amount of available memory in bytes."""
        model_input_time: builtins.float
        """Time between two consecutive `GetNext` calls to the iterator represented
        by the output node.
        """
        def __init__(
            self,
            *,
            algorithm: global___AutotuneAlgorithm.ValueType | None = ...,
            cpu_budget: builtins.int | None = ...,
            ram_budget: builtins.int | None = ...,
            model_input_time: builtins.float | None = ...,
        ) -> None: ...
        def ClearField(
            self,
            field_name: typing.Literal[
                "algorithm",
                b"algorithm",
                "cpu_budget",
                b"cpu_budget",
                "model_input_time",
                b"model_input_time",
                "ram_budget",
                b"ram_budget",
            ],
        ) -> None: ...

    DATASET_NAME_FIELD_NUMBER: builtins.int
    NODES_FIELD_NUMBER: builtins.int
    OUTPUT_FIELD_NUMBER: builtins.int
    ID_COUNTER_FIELD_NUMBER: builtins.int
    OPTIMIZATION_PARAMS_FIELD_NUMBER: builtins.int
    GAP_TIMES_FIELD_NUMBER: builtins.int
    dataset_name: builtins.str
    """User-defined name for the dataset. Empty if no name was set."""
    output: builtins.int
    """ID of the output node of this model."""
    id_counter: builtins.int
    """Counter for node IDs of this model."""
    @property
    def nodes(self) -> google.protobuf.internal.containers.MessageMap[builtins.int, global___ModelProto.Node]:
        """Map of node IDs to nodes of this model."""

    @property
    def optimization_params(self) -> global___ModelProto.OptimizationParams: ...
    @property
    def gap_times(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]: ...
    def __init__(
        self,
        *,
        dataset_name: builtins.str | None = ...,
        nodes: collections.abc.Mapping[builtins.int, global___ModelProto.Node] | None = ...,
        output: builtins.int | None = ...,
        id_counter: builtins.int | None = ...,
        optimization_params: global___ModelProto.OptimizationParams | None = ...,
        gap_times: collections.abc.Iterable[builtins.int] | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["optimization_params", b"optimization_params"]) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "dataset_name",
            b"dataset_name",
            "gap_times",
            b"gap_times",
            "id_counter",
            b"id_counter",
            "nodes",
            b"nodes",
            "optimization_params",
            b"optimization_params",
            "output",
            b"output",
        ],
    ) -> None: ...

global___ModelProto = ModelProto



#!/usr/bin/env python3

import os
import shutil
import subprocess
import sys
from pathlib import Path

from ts_utils.utils import parse_requirements, print_command

_WELL_KNOWN_FILE = Path("tests", "pyright_test.py")


def main() -> None:
    if not _WELL_KNOWN_FILE.exists():
        print("pyright_test.py must be run from the typeshed root directory", file=sys.stderr)
        sys.exit(1)

    # subprocess.run on Windows does not look in PATH.
    npx = shutil.which("npx")

    if npx is None:
        print("error finding npx; is Node.js installed?", file=sys.stderr)
        sys.exit(1)

    try:
        subprocess.run([npx, "--version"], check=False)
    except OSError:
        print("error running npx; is Node.js installed?", file=sys.stderr)
        sys.exit(1)

    req = parse_requirements()["pyright"]
    spec = str(req.specifier)
    pyright_version = spec[2:]

    # TODO: We're currently using npx to run pyright, instead of calling the
    # version installed into the virtual environment, due to failures on some
    # platforms. https://github.com/python/typeshed/issues/11614
    os.environ["PYRIGHT_PYTHON_FORCE_VERSION"] = pyright_version
    command = [npx, f"pyright@{pyright_version}", *sys.argv[1:]]
    print_command(command)

    ret = subprocess.run(command, check=False).returncode
    sys.exit(ret)


if __name__ == "__main__":
    main()



from typing import Any, ClassVar
from typing_extensions import Self

import passlib.utils.handlers as uh
from passlib.handlers.bcrypt import _wrapped_bcrypt
from passlib.ifc import DisabledHash

class DjangoSaltedHash(uh.HasSalt, uh.GenericHandler):
    default_salt_size: ClassVar[int]
    salt_chars: ClassVar[str]
    checksum_chars: ClassVar[str]
    @classmethod
    def from_string(cls, hash) -> Self: ...  # type: ignore[override]

class DjangoVariableHash(uh.HasRounds, DjangoSaltedHash):  # type: ignore[misc]
    min_rounds: ClassVar[int]
    @classmethod
    def from_string(cls, hash) -> Self: ...  # type: ignore[override]

class django_salted_sha1(DjangoSaltedHash):
    name: ClassVar[str]
    django_name: ClassVar[str]
    ident: ClassVar[str]
    checksum_size: ClassVar[int]

class django_salted_md5(DjangoSaltedHash):
    name: ClassVar[str]
    django_name: ClassVar[str]
    ident: ClassVar[str]
    checksum_size: ClassVar[int]

django_bcrypt: Any

class django_bcrypt_sha256(_wrapped_bcrypt):
    name: ClassVar[str]
    django_name: ClassVar[str]
    django_prefix: ClassVar[str]
    @classmethod
    def identify(cls, hash): ...
    @classmethod
    def from_string(cls, hash) -> Self: ...  # type: ignore[override]

class django_pbkdf2_sha256(DjangoVariableHash):
    name: ClassVar[str]
    django_name: ClassVar[str]
    ident: ClassVar[str]
    min_salt_size: ClassVar[int]
    max_rounds: ClassVar[int]
    checksum_chars: ClassVar[str]
    checksum_size: ClassVar[int]
    default_rounds: ClassVar[int]

class django_pbkdf2_sha1(django_pbkdf2_sha256):
    name: ClassVar[str]
    django_name: ClassVar[str]
    ident: ClassVar[str]
    checksum_size: ClassVar[int]
    default_rounds: ClassVar[int]

django_argon2: Any

class django_des_crypt(uh.TruncateMixin, uh.HasSalt, uh.GenericHandler):  # type: ignore[misc]
    name: ClassVar[str]
    django_name: ClassVar[str]
    ident: ClassVar[str]
    checksum_chars: ClassVar[str]
    salt_chars: ClassVar[str]
    checksum_size: ClassVar[int]
    min_salt_size: ClassVar[int]
    default_salt_size: ClassVar[int]
    truncate_size: ClassVar[int]
    use_duplicate_salt: bool
    @classmethod
    def from_string(cls, hash) -> Self: ...  # type: ignore[override]

class django_disabled(DisabledHash, uh.StaticHandler):
    name: ClassVar[str]
    suffix_length: ClassVar[int]
    @classmethod
    def identify(cls, hash: str | bytes) -> bool: ...
    @classmethod
    def verify(cls, secret: str | bytes, hash: str | bytes) -> bool: ...  # type: ignore[override]

__all__ = [
    "django_salted_sha1",
    "django_salted_md5",
    "django_bcrypt",
    "django_pbkdf2_sha1",
    "django_pbkdf2_sha256",
    "django_argon2",
    "django_des_crypt",
    "django_disabled",
]



from collections.abc import Collection
from typing_extensions import Self

from .base import BaseMultipartGeometry
from .collection import GeometryCollection
from .point import Point, _PointLike

__all__ = ["MultiPoint"]

class MultiPoint(BaseMultipartGeometry[Point]):
    # Note on "points" type in `__new__`:
    # * `Collection` here is loose as the expected type should support "__getitem__".
    # * `Sequence` is more correct but it will lead to False positives with common types
    #   like np.ndarray, pd.Index, pd.Series, ...
    # I went with Collection as false negatives seem better to me than false positives in this case
    __slots__: list[str] = []
    def __new__(self, points: MultiPoint | Collection[_PointLike] | None = None) -> Self: ...
    def svg(self, scale_factor: float = 1.0, fill_color: str | None = None, opacity: float | None = None) -> str: ...  # type: ignore[override]
    # more precise base overrides
    @property
    def boundary(self) -> GeometryCollection: ...  # empty geometry collection



from typing import Any

from hvac.api.system_backend.system_backend_mixin import SystemBackendMixin
from requests import Response

class Raft(SystemBackendMixin):
    def join_raft_cluster(
        self, leader_api_addr, retry: bool = False, leader_ca_cert=None, leader_client_cert=None, leader_client_key=None
    ): ...
    def read_raft_config(self): ...
    def remove_raft_node(self, server_id): ...
    def take_raft_snapshot(self): ...
    def restore_raft_snapshot(self, snapshot): ...
    def force_restore_raft_snapshot(self, snapshot): ...
    def read_raft_auto_snapshot_status(self, name: str) -> Response: ...
    def read_raft_auto_snapshot_config(self, name: str) -> Response: ...
    def list_raft_auto_snapshot_configs(self) -> Response: ...
    def create_or_update_raft_auto_snapshot_config(
        self, name: str, interval: str, storage_type: str, retain: int = 1, **kwargs: Any
    ) -> Response: ...
    def delete_raft_auto_snapshot_config(self, name: str) -> Response: ...



from abc import ABC, abstractmethod
from collections.abc import Sequence
from dataclasses import dataclass
from enum import Enum, Flag, IntEnum, IntFlag
from typing import Final, Literal
from typing_extensions import Self, TypeAlias

from .drawing import DeviceCMYK, DeviceGray, DeviceRGB
from .syntax import Name

_Color: TypeAlias = str | int | Sequence[int] | DeviceCMYK | DeviceGray | DeviceRGB

class SignatureFlag(IntEnum):
    SIGNATURES_EXIST = 1
    APPEND_ONLY = 2

class CoerciveEnum(Enum):  # type: ignore[misc]  # Enum with no members
    @classmethod
    def coerce(cls, value: Self | str, case_sensitive: bool = False) -> Self: ...

class CoerciveIntEnum(IntEnum):  # type: ignore[misc]  # Enum with no members
    @classmethod
    def coerce(cls, value: Self | str | int) -> Self: ...

class CoerciveIntFlag(IntFlag):  # type: ignore[misc]  # Enum with no members
    @classmethod
    def coerce(cls, value: Self | str | int) -> Self: ...

class WrapMode(CoerciveEnum):
    WORD = "WORD"
    CHAR = "CHAR"

class CharVPos(CoerciveEnum):
    SUP = "SUP"
    SUB = "SUB"
    NOM = "NOM"
    DENOM = "DENOM"
    LINE = "LINE"

class Align(CoerciveEnum):
    C = "CENTER"
    X = "X_CENTER"
    L = "LEFT"
    R = "RIGHT"
    J = "JUSTIFY"

    @classmethod
    def coerce(cls, value: Self | str) -> Self: ...  # type: ignore[override]

_Align: TypeAlias = Align | Literal["CENTER", "X_CENTER", "LEFT", "RIGHT", "JUSTIFY"]  # noqa: Y047

class VAlign(CoerciveEnum):
    M = "MIDDLE"
    T = "TOP"
    B = "BOTTOM"

    @classmethod
    def coerce(cls, value: Self | str) -> Self: ...  # type: ignore[override]

class TextEmphasis(CoerciveIntFlag):
    NONE = 0
    B = 1
    I = 2
    U = 4
    S = 8

    @property
    def style(self) -> str: ...
    def add(self, value: TextEmphasis) -> TextEmphasis: ...
    def remove(self, value: TextEmphasis) -> TextEmphasis: ...

class MethodReturnValue(CoerciveIntFlag):
    PAGE_BREAK = 1
    LINES = 2
    HEIGHT = 4

class CellBordersLayout(CoerciveIntFlag):
    NONE = 0
    LEFT = 1
    RIGHT = 2
    TOP = 4
    BOTTOM = 8
    ALL = 15
    INHERIT = 16

@dataclass
class TableBorderStyle:
    thickness: float | None = None
    color: int | tuple[int, int, int] | None = None
    dash: float | None = None
    gap: float = 0.0
    phase: float = 0.0

    @staticmethod
    def from_bool(should_draw: TableBorderStyle | bool | None) -> TableBorderStyle: ...
    @property
    def dash_dict(self) -> dict[str, float | None]: ...
    def changes_stroke(self, pdf) -> bool: ...
    def should_render(self) -> bool: ...
    def get_change_stroke_commands(self, scale: float) -> list[str]: ...
    @staticmethod
    def get_line_command(x1: float, y1: float, x2: float, y2: float) -> list[str]: ...
    def get_draw_commands(self, pdf, x1: float, y1: float, x2: float, y2: float) -> list[str]: ...

@dataclass
class TableCellStyle:
    left: bool | TableBorderStyle = False
    bottom: bool | TableBorderStyle = False
    right: bool | TableBorderStyle = False
    top: bool | TableBorderStyle = False

    @staticmethod
    def get_change_fill_color_command(color: _Color | None) -> list[str]: ...
    def get_draw_commands(
        self, pdf, x1: float, y1: float, x2: float, y2: float, fill_color: _Color | None = None
    ) -> list[str]: ...
    def override_cell_border(self, cell_border: CellBordersLayout) -> Self: ...
    def draw_cell_border(self, pdf, x1: float, y1: float, x2: float, y2: float, fill_color: _Color | None = None) -> None: ...

class TableBordersLayout(ABC):
    ALL: Final[TableBordersLayoutAll]
    NONE: Final[TableBordersLayoutNone]
    INTERNAL: Final[TableBordersLayoutInternal]
    MINIMAL: Final[TableBordersLayoutMinimal]
    HORIZONTAL_LINES: Final[TableBordersLayoutHorizontalLines]
    NO_HORIZONTAL_LINES: Final[TableBordersLayoutNoHorizontalLines]
    SINGLE_TOP_LINE: Final[TableBordersLayoutSingleTopLine]
    @abstractmethod
    def cell_style_getter(
        self, row_idx: int, col_idx: int, col_pos: int, num_heading_rows: int, num_rows: int, num_col_idx: int, num_col_pos: int
    ) -> TableCellStyle: ...
    @classmethod
    def coerce(cls, value: Self | str) -> Self: ...

class TableBordersLayoutAll(TableBordersLayout):
    def cell_style_getter(
        self, row_idx: int, col_idx: int, col_pos: int, num_heading_rows: int, num_rows: int, num_col_idx: int, num_col_pos: int
    ) -> TableCellStyle: ...

class TableBordersLayoutNone(TableBordersLayout):
    def cell_style_getter(
        self, row_idx: int, col_idx: int, col_pos: int, num_heading_rows: int, num_rows: int, num_col_idx: int, num_col_pos: int
    ) -> TableCellStyle: ...

class TableBordersLayoutInternal(TableBordersLayout):
    def cell_style_getter(
        self, row_idx: int, col_idx: int, col_pos: int, num_heading_rows: int, num_rows: int, num_col_idx: int, num_col_pos: int
    ) -> TableCellStyle: ...

class TableBordersLayoutMinimal(TableBordersLayout):
    def cell_style_getter(
        self, row_idx: int, col_idx: int, col_pos: int, num_heading_rows: int, num_rows: int, num_col_idx: int, num_col_pos: int
    ) -> TableCellStyle: ...

class TableBordersLayoutHorizontalLines(TableBordersLayout):
    def cell_style_getter(
        self, row_idx: int, col_idx: int, col_pos: int, num_heading_rows: int, num_rows: int, num_col_idx: int, num_col_pos: int
    ) -> TableCellStyle: ...

class TableBordersLayoutNoHorizontalLines(TableBordersLayout):
    def cell_style_getter(
        self, row_idx: int, col_idx: int, col_pos: int, num_heading_rows: int, num_rows: int, num_col_idx: int, num_col_pos: int
    ) -> TableCellStyle: ...

class TableBordersLayoutSingleTopLine(TableBordersLayout):
    def cell_style_getter(
        self, row_idx: int, col_idx: int, col_pos: int, num_heading_rows: int, num_rows: int, num_col_idx: int, num_col_pos: int
    ) -> TableCellStyle: ...

class TableCellFillMode(CoerciveEnum):
    NONE = "NONE"
    ALL = "ALL"
    ROWS = "ROWS"
    COLUMNS = "COLUMNS"
    EVEN_ROWS = "EVEN_ROWS"
    EVEN_COLUMNS = "EVEN_COLUMNS"

    def should_fill_cell(self, i: int, j: int) -> bool: ...
    @classmethod
    def coerce(cls, value: Self | str) -> Self: ...  # type: ignore[override]

class TableSpan(CoerciveEnum):
    ROW = "ROW"
    COL = "COL"

class TableHeadingsDisplay(CoerciveIntEnum):
    NONE = 0
    ON_TOP_OF_EVERY_PAGE = 1

class RenderStyle(CoerciveEnum):
    D = "DRAW"
    F = "FILL"
    DF = "DRAW_FILL"
    @property
    def operator(self) -> str: ...
    @property
    def is_draw(self) -> bool: ...
    @property
    def is_fill(self) -> bool: ...
    @classmethod
    def coerce(cls, value: Self | str) -> Self: ...  # type: ignore[override]

class TextMode(CoerciveIntEnum):
    FILL = 0
    STROKE = 1
    FILL_STROKE = 2
    INVISIBLE = 3
    FILL_CLIP = 4
    STROKE_CLIP = 5
    FILL_STROKE_CLIP = 6
    CLIP = 7

class XPos(CoerciveEnum):
    LEFT = "LEFT"
    RIGHT = "RIGHT"
    START = "START"
    END = "END"
    WCONT = "WCONT"
    CENTER = "CENTER"
    LMARGIN = "LMARGIN"
    RMARGIN = "RMARGIN"

class YPos(CoerciveEnum):
    TOP = "TOP"
    LAST = "LAST"
    NEXT = "NEXT"
    TMARGIN = "TMARGIN"
    BMARGIN = "BMARGIN"

class Angle(CoerciveIntEnum):
    NORTH = 90
    EAST = 0
    SOUTH = 270
    WEST = 180
    NORTHEAST = 45
    SOUTHEAST = 315
    SOUTHWEST = 225
    NORTHWEST = 135

class PageLayout(CoerciveEnum):
    SINGLE_PAGE = Name("SinglePage")
    ONE_COLUMN = Name("OneColumn")
    TWO_COLUMN_LEFT = Name("TwoColumnLeft")
    TWO_COLUMN_RIGHT = Name("TwoColumnRight")
    TWO_PAGE_LEFT = Name("TwoPageLeft")
    TWO_PAGE_RIGHT = Name("TwoPageRight")

class PageMode(CoerciveEnum):
    USE_NONE = Name("UseNone")
    USE_OUTLINES = Name("UseOutlines")
    USE_THUMBS = Name("UseThumbs")
    FULL_SCREEN = Name("FullScreen")
    USE_OC = Name("UseOC")
    USE_ATTACHMENTS = Name("UseAttachments")

class TextMarkupType(CoerciveEnum):
    HIGHLIGHT = Name("Highlight")
    UNDERLINE = Name("Underline")
    SQUIGGLY = Name("Squiggly")
    STRIKE_OUT = Name("StrikeOut")

class BlendMode(CoerciveEnum):
    NORMAL = Name("Normal")
    MULTIPLY = Name("Multiply")
    SCREEN = Name("Screen")
    OVERLAY = Name("Overlay")
    DARKEN = Name("Darken")
    LIGHTEN = Name("Lighten")
    COLOR_DODGE = Name("ColorDodge")
    COLOR_BURN = Name("ColorBurn")
    HARD_LIGHT = Name("HardLight")
    SOFT_LIGHT = Name("SoftLight")
    DIFFERENCE = Name("Difference")
    EXCLUSION = Name("Exclusion")
    HUE = Name("Hue")
    SATURATION = Name("Saturation")
    COLOR = Name("Color")
    LUMINOSITY = Name("Luminosity")

class AnnotationFlag(CoerciveIntEnum):
    INVISIBLE = 1
    HIDDEN = 2
    PRINT = 4
    NO_ZOOM = 8
    NO_ROTATE = 16
    NO_VIEW = 32
    READ_ONLY = 64
    LOCKED = 128
    TOGGLE_NO_VIEW = 256
    LOCKED_CONTENTS = 512

class AnnotationName(CoerciveEnum):
    NOTE = Name("Note")
    COMMENT = Name("Comment")
    HELP = Name("Help")
    PARAGRAPH = Name("Paragraph")
    NEW_PARAGRAPH = Name("NewParagraph")
    INSERT = Name("Insert")

class FileAttachmentAnnotationName(CoerciveEnum):
    PUSH_PIN = Name("PushPin")
    GRAPH_PUSH_PIN = Name("GraphPushPin")
    PAPERCLIP_TAG = Name("PaperclipTag")

class IntersectionRule(CoerciveEnum):
    NONZERO = "nonzero"
    EVENODD = "evenodd"

class PathPaintRule(CoerciveEnum):
    STROKE = "S"
    FILL_NONZERO = "f"
    FILL_EVENODD = "f*"
    STROKE_FILL_NONZERO = "B"
    STROKE_FILL_EVENODD = "B*"
    DONT_PAINT = "n"
    AUTO = "auto"

class ClippingPathIntersectionRule(CoerciveEnum):
    NONZERO = "W"
    EVENODD = "W*"

class StrokeCapStyle(CoerciveIntEnum):
    BUTT = 0
    ROUND = 1
    SQUARE = 2

class StrokeJoinStyle(CoerciveIntEnum):
    MITER = 0
    ROUND = 1
    BEVEL = 2

class PDFStyleKeys(Enum):
    FILL_ALPHA = Name("ca")
    BLEND_MODE = Name("BM")
    STROKE_ALPHA = Name("CA")
    STROKE_ADJUSTMENT = Name("SA")
    STROKE_WIDTH = Name("LW")
    STROKE_CAP_STYLE = Name("LC")
    STROKE_JOIN_STYLE = Name("LJ")
    STROKE_MITER_LIMIT = Name("ML")
    STROKE_DASH_PATTERN = Name("D")

class Corner(CoerciveEnum):
    TOP_RIGHT = "TOP_RIGHT"
    TOP_LEFT = "TOP_LEFT"
    BOTTOM_RIGHT = "BOTTOM_RIGHT"
    BOTTOM_LEFT = "BOTTOM_LEFT"

class FontDescriptorFlags(Flag):
    FIXED_PITCH = 1
    SYMBOLIC = 4
    ITALIC = 64
    FORCE_BOLD = 262144

class AccessPermission(IntFlag):
    PRINT_LOW_RES = 4
    MODIFY = 8
    COPY = 16
    ANNOTATION = 32
    FILL_FORMS = 256
    COPY_FOR_ACCESSIBILITY = 512
    ASSEMBLE = 1024
    PRINT_HIGH_RES = 2048
    @classmethod
    def all(cls) -> int: ...
    @classmethod
    def none(cls) -> Literal[0]: ...

class EncryptionMethod(Enum):
    NO_ENCRYPTION = 0
    RC4 = 1
    AES_128 = 2
    AES_256 = 3

class TextDirection(CoerciveEnum):
    LTR = "LTR"
    RTL = "RTL"
    TTB = "TTB"
    BTT = "BTT"

class OutputIntentSubType(CoerciveEnum):
    PDFX = "GTS_PDFX"
    PDFA = "GTS_PDFA1"
    ISOPDF = "ISO_PDFE1"

class PageLabelStyle(CoerciveEnum):
    NUMBER = "D"
    UPPER_ROMAN = "R"
    LOWER_ROMAN = "r"
    UPPER_LETTER = "A"
    LOWER_LETTER = "a"
    NONE = None

class Duplex(CoerciveEnum):
    SIMPLEX = "Simplex"
    DUPLEX_FLIP_SHORT_EDGE = "DuplexFlipShortEdge"
    DUPLEX_FLIP_LONG_EDGE = "DuplexFlipLongEdge"

class PageBoundaries(CoerciveEnum):
    ART_BOX = "ArtBox"
    BLEED_BOX = "BleedBox"
    CROP_BOX = "CropBox"
    MEDIA_BOX = "MediaBox"
    TRIM_BOX = "TrimBox"

class PageOrientation(CoerciveEnum):
    PORTRAIT = "P"
    LANDSCAPE = "L"

    @classmethod
    def coerce(cls, value: Self | str) -> Self: ...  # type: ignore[override]

class PDFResourceType(Enum):
    EXT_G_STATE = "ExtGState"
    COLOR_SPACE = "ColorSpace"
    PATTERN = "Pattern"
    SHADDING = "Shading"
    X_OBJECT = "XObject"
    FONT = "Font"
    PROC_SET = "ProcSet"
    PROPERTIES = "Properties"



from typing import Any
from typing_extensions import Self

__tracebackhide__: bool

class ContainsMixin:
    def contains(self, *items: Any) -> Self: ...
    def does_not_contain(self, *items: Any) -> Self: ...
    def contains_only(self, *items: Any) -> Self: ...
    def contains_sequence(self, *items: Any) -> Self: ...
    def contains_duplicates(self) -> Self: ...
    def does_not_contain_duplicates(self) -> Self: ...
    def is_empty(self) -> Self: ...
    def is_not_empty(self) -> Self: ...
    def is_in(self, *items: Any) -> Self: ...
    def is_not_in(self, *items: Any) -> Self: ...



from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = ["stoer_wagner"]

@_dispatchable
def stoer_wagner(G: Graph[_Node], weight: str = "weight", heap: type = ...): ...



import re
from _typeshed import Incomplete
from collections.abc import Iterable
from typing_extensions import deprecated

from . import base

__all__ = ["Filter"]

allowed_elements: frozenset[tuple[str, str]]
allowed_attributes: frozenset[tuple[None, str] | tuple[str, str]]
attr_val_is_uri: frozenset[tuple[None, str] | tuple[str, str]]
svg_attr_val_allows_ref: frozenset[tuple[None, str]]
svg_allow_local_href: frozenset[tuple[None, str]]
allowed_css_properties: frozenset[str]
allowed_css_keywords: frozenset[str]
allowed_svg_properties: frozenset[str]
allowed_protocols: frozenset[str]
allowed_content_types: frozenset[str]
data_content_type: re.Pattern[str]

@deprecated("html5lib's sanitizer is deprecated; see https://github.com/html5lib/html5lib-python/issues/443")
class Filter(base.Filter[dict[str, Incomplete]]):
    allowed_elements: Iterable[tuple[str | None, str]]
    allowed_attributes: Iterable[tuple[str | None, str]]
    allowed_css_properties: Iterable[str]
    allowed_css_keywords: Iterable[str]
    allowed_svg_properties: Iterable[str]
    allowed_protocols: Iterable[str]
    allowed_content_types: Iterable[str]
    attr_val_is_uri: Iterable[tuple[str | None, str]]
    svg_attr_val_allows_ref: Iterable[tuple[str | None, str]]
    svg_allow_local_href: Iterable[tuple[str | None, str]]
    def __init__(
        self,
        source: Iterable[dict[str, Incomplete]],
        allowed_elements: Iterable[tuple[str | None, str]] = ...,
        allowed_attributes: Iterable[tuple[str | None, str]] = ...,
        allowed_css_properties: Iterable[str] = ...,
        allowed_css_keywords: Iterable[str] = ...,
        allowed_svg_properties: Iterable[str] = ...,
        allowed_protocols: Iterable[str] = ...,
        allowed_content_types: Iterable[str] = ...,
        attr_val_is_uri: Iterable[tuple[str | None, str]] = ...,
        svg_attr_val_allows_ref: Iterable[tuple[str | None, str]] = ...,
        svg_allow_local_href: Iterable[tuple[str | None, str]] = ...,
    ) -> None: ...
    def sanitize_token(self, token: dict[str, Incomplete]) -> dict[str, Incomplete] | None: ...
    def allowed_token(self, token: dict[str, Incomplete]) -> dict[str, Incomplete]: ...
    def disallowed_token(self, token: dict[str, Incomplete]) -> dict[str, Incomplete]: ...
    def sanitize_css(self, style: str) -> str: ...
